@@ -4,7 +4,8 @@ import enum
 import functools
 import platform
 import struct
-from typing import Self, Any
+from typing import Any
+from pathlib import Path
 
 # faster than builtin re
 import regex
@@ -13,14 +14,14 @@ import regex
 class Process:
     """A connected process"""
 
-    @property
+    @functools.cached_property
     def process_64_bit(self) -> bool:
         """
         If this process is 64 bit
         """
         raise NotImplementedError()
 
-    @property
+    @functools.cached_property
     def python_64_bit(self) -> bool:
         """
         If the current python is 64 bit
@@ -28,13 +29,20 @@ class Process:
         # we can just check the pointer size; 4 = 32, 8 = 64
         return ctypes.sizeof(ctypes.c_void_p) == 8
 
-    @property
+    @functools.cached_property
     def system_64_bit(self) -> bool:
         """
         If the system is 64 bit
         """
         return platform.architecture()[0] == "64bit"
 
+    @functools.cached_property
+    def executable_path(self) -> Path:
+        """
+        Path to the process's executable
+        """
+        raise NotImplementedError()
+
     @functools.cached_property
     def pointer_format_string(self) -> str:
         if self.process_64_bit:
@@ -50,7 +58,7 @@ class Process:
             return 4
 
     @classmethod
-    def from_name(cls, name: str) -> Self:
+    def from_name(cls, name: str) -> "Process":
         """
         Open a process by name
 
@@ -63,7 +71,7 @@ class Process:
         raise NotImplementedError()
 
     @classmethod
-    def from_id(cls, pid: int) -> Self:
+    def from_id(cls, pid: int) -> "Process":
         """
         Open a process by id
 
@@ -212,6 +220,15 @@ class WindowsMemoryBasicInformation(ctypes.Structure):
         ]
 
 
+# https://learn.microsoft.com/en-us/windows/win32/api/psapi/ns-psapi-moduleinfo
+class WindowsModuleInfo(ctypes.Structure):
+    _fields_ = [
+        ("lpBaseOfDll", ctypes.wintypes.LPVOID),
+        ("SizeOfImage", ctypes.wintypes.DWORD),
+        ("EntryPoint", ctypes.wintypes.LPVOID),
+    ]
+
+
 # https://learn.microsoft.com/en-us/windows/win32/memory/memory-protection-constants
 class WindowsMemoryProtection(enum.IntFlag):
     # note: can't read
@@ -221,7 +238,7 @@ class WindowsMemoryProtection(enum.IntFlag):
     PAGE_READWRITE = 0x4
     # note: wacky on write
     PAGE_WRITECOPY = 0x8
-    # note: can't write, can read?
+    # note: can't write, can't read
     PAGE_EXECUTE = 0x10
     # note: can't write
     PAGE_EXECUTE_READ = 0x20
@@ -345,8 +362,22 @@ class WindowsProcess(Process):
         # this is the only case where the process is 64 bit
         return self.system_64_bit and wow_64_process.value == 0
 
+    @functools.cached_property
+    def executable_path(self) -> Path:
+        with CheckWindowsOsError():
+            # https://learn.microsoft.com/en-us/windows/win32/api/libloaderapi/nf-libloaderapi-getmodulefilenamea
+            file_name = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
+
+            ctypes.windll.kernel32.GetModuleFileNameW(
+                0,
+                ctypes.byref(file_name),
+                ctypes.wintypes.MAX_PATH,
+            )
+
+        return Path(file_name.value)
+
     @classmethod
-    def from_name(cls, name: str, *, require_debug: bool = True) -> Self:
+    def from_name(cls, name: str, *, require_debug: bool = True) -> "WindowsProcess":
         import ctypes.wintypes
 
         # https://learn.microsoft.com/en-us/windows/win32/api/tlhelp32/ns-tlhelp32-processentry32
@@ -403,7 +434,7 @@ class WindowsProcess(Process):
         raise ValueError(f"No processes found named {name}; make sure you included the .exe and any capital letters")
 
     @classmethod
-    def from_id(cls, pid: int, *, require_debug: bool = True) -> Self:
+    def from_id(cls, pid: int, *, require_debug: bool = True) -> "WindowsProcess":
         try:
             cls._get_debug_privileges()
         except OSError as error:
@@ -568,6 +599,65 @@ class WindowsProcess(Process):
 
         return memory_basic_information
 
+    # TODO: check if you actually can't get modules on linux
+    # note: platform dependent
+    def get_modules(self, base_only: bool = False) -> list[WindowsModuleInfo] | WindowsModuleInfo:
+        # TODO: for some reason EnumProcessModulesEx always sets LastError?
+        # with CheckWindowsOsError():
+        # TODO: is it always the psapi dll? check requirments section
+        # https://learn.microsoft.com/en-us/windows/win32/api/psapi/nf-psapi-enumprocessmodulesex
+        lpcb_needed = ctypes.wintypes.DWORD()
+
+        success = ctypes.windll.psapi.EnumProcessModulesEx(
+            self.process_handle,
+            0,
+            0,
+            ctypes.byref(lpcb_needed),
+            0x3,
+        )
+
+        if success == 0:
+            raise RuntimeError("EnumProcessModulesEx (get size) failed")
+
+        # with CheckWindowsOsError():
+        module_handles_type = ctypes.wintypes.HMODULE * (
+                lpcb_needed.value // ctypes.sizeof(ctypes.wintypes.HMODULE)
+        )
+        module_handles = module_handles_type()
+
+        success = ctypes.windll.psapi.EnumProcessModulesEx(
+            self.process_handle,
+            ctypes.byref(module_handles),
+            lpcb_needed,
+            ctypes.byref(ctypes.wintypes.DWORD()),
+            0x3,
+        )
+
+        if success == 0:
+            raise RuntimeError("EnumProcessModulesEx failed")
+
+        with CheckWindowsOsError():
+            modules = []
+            for module_handle in module_handles:
+                # https://learn.microsoft.com/en-us/windows/win32/api/psapi/nf-psapi-getmoduleinformation
+                module_info = WindowsModuleInfo()
+
+                success = ctypes.windll.psapi.GetModuleInformation(
+                    self.process_handle,
+                    ctypes.wintypes.HMODULE(module_handle),
+                    ctypes.byref(module_info),
+                    ctypes.sizeof(module_info),
+                )
+
+                if success == 0:
+                    raise ValueError(f"GetModuleInformation failed for handle {module_handle}")
+
+                if base_only:
+                    return module_info
+
+                modules.append(module_info)
+
+        return modules
 
 # TODO
 # class LinuxProcess(Process):
