@@ -13,14 +13,7 @@ def is_aligned(value: int, align: int) -> bool:
     return (value & (align - 1)) == 0
 
 
-def is_power_of_two(value: int) -> bool:
-    return value > 0 and is_aligned(value, value)
-
-
 def get_alignment(size: int) -> int:
-    if is_power_of_two(size):
-        raise ValueError("alignment cannot be guessed")
-
     return size & -size
 
 
